@@ -2,42 +2,26 @@
 Load snippets from source files to show violation lines
 in HTML reports.
 """
-from __future__ import unicode_literals
+import chardet
 import pygments
 from pygments.formatters.html import HtmlFormatter
+from pygments.formatters.terminal import TerminalFormatter
 from pygments.lexers import guess_lexer_for_filename
 from pygments.lexers.special import TextLexer
 from pygments.util import ClassNotFound
-import six
 
 from diff_cover.git_path import GitPathTool
-# If tokenize.open (Python>=3.2) is available, use that to open python files.
-# Otherwise, use detect_encoding from lib2to3 (2.6, 2.7) wrapped the same way
-# that tokenize.open would have.
-try:
-    from tokenize import open as openpy
-except ImportError:
-    def openpy(filename):
-        from lib2to3.pgen2.tokenize import detect_encoding
-        import io
 
-        # The following is copied from tokenize.py in Python 3.2,
-        # Copyright (c) 2001-2014 Python Software Foundation; All Rights Reserved
-        buffer = io.open(filename, 'rb')
-        encoding, lines = detect_encoding(buffer.readline)
-        buffer.seek(0)
-        text = io.TextIOWrapper(buffer, encoding, line_buffering=True)
-        text.mode = 'r'
-        return text
+from tokenize import open as openpy
 
 
-class Snippet(object):
+class Snippet:
     """
     A source code snippet.
     """
 
-    VIOLATION_COLOR = '#ffcccc'
-    DIV_CSS_CLASS = 'snippet'
+    VIOLATION_COLOR = "#ffcccc"
+    DIV_CSS_CLASS = "snippet"
 
     # Number of extra lines to include before and after
     # each snippet to provide context.
@@ -48,8 +32,23 @@ class Snippet(object):
     # should split into two snippets.
     MAX_GAP_IN_SNIPPET = 4
 
-    def __init__(self, src_tokens, src_filename,
-                 start_line, violation_lines):
+    # See https://github.com/github/linguist/blob/master/lib/linguist/languages.yml
+    # for typical values of accepted programming language hints in Markdown code fenced blocks
+    LEXER_TO_MARKDOWN_CODE_HINT = {
+        "Python": "python",
+        "C++": "cpp",
+        # TODO: expand this list...
+    }
+
+    def __init__(
+        self,
+        src_tokens,
+        src_filename,
+        start_line,
+        last_line,
+        violation_lines,
+        lexer_name,
+    ):
         """
         Create a source code snippet.
 
@@ -64,18 +63,27 @@ class Snippet(object):
         in `src_str`.  The first line in the file is
         line number 1.
 
+        `last_line` is the line number of last line
+        in `src_str`.
+
         `violation_lines` is a list of line numbers
         to highlight as violations.
 
+        `lexer_name` provides an hint on the
+        programming language for this snippet.
+        See https://pygments.org/docs/lexers/
+
         Raises a `ValueError` if `start_line` is less than 1
         """
         if start_line < 1:
-            raise ValueError('Start line must be >= 1')
+            raise ValueError("Start line must be >= 1")
 
         self._src_tokens = src_tokens
         self._src_filename = src_filename
         self._start_line = start_line
+        self._last_line = last_line
         self._violation_lines = violation_lines
+        self._lexer_name = lexer_name
 
     @classmethod
     def style_defs(cls):
@@ -95,11 +103,38 @@ class Snippet(object):
             cssclass=self.DIV_CSS_CLASS,
             linenos=True,
             linenostart=self._start_line,
-            hl_lines=self._shift_lines(
-                self._violation_lines,
-                self._start_line
-            ),
-            lineanchors=self._src_filename
+            hl_lines=self._shift_lines(self._violation_lines, self._start_line),
+            lineanchors=self._src_filename,
+        )
+
+        return pygments.format(self.src_tokens(), formatter)
+
+    def markdown(self):
+        """
+        Return a Markdown representation of the snippet using Markdown fenced code blocks.
+        See https://github.github.com/gfm/#fenced-code-blocks.
+        """
+        header = "Lines %d-%d\n\n" % (self._start_line, self._last_line)
+        if self._lexer_name in self.LEXER_TO_MARKDOWN_CODE_HINT:
+            return header + (
+                "```"
+                + self.LEXER_TO_MARKDOWN_CODE_HINT[self._lexer_name]
+                + "\n"
+                + self.text()
+                + "\n```\n"
+            )
+
+        # unknown programming language, return a non-decorated fenced code block:
+        return "```\n" + self.text() + "\n```\n"
+
+    def terminal(self):
+        """
+        Return a Terminal-friendly (with ANSI color sequences) representation of the snippet.
+        """
+        formatter = TerminalFormatter(
+            linenos=True,
+            colorscheme=None,
+            linenostart=self._start_line,
         )
 
         return pygments.format(self.src_tokens(), formatter)
@@ -116,7 +151,7 @@ class Snippet(object):
         Return a tuple of the form `(start_line, end_line)`
         indicating the start and end line number of the snippet.
         """
-        num_lines = len(self.text().split('\n'))
+        num_lines = len(self.text().split("\n"))
         end_line = self._start_line + num_lines - 1
         return (self._start_line, end_line)
 
@@ -124,18 +159,57 @@ class Snippet(object):
         """
         Return the source text for the snippet.
         """
-        return ''.join([val for _, val in self._src_tokens])
+        return "".join([val for _, val in self._src_tokens])
 
     @classmethod
-    def load_snippets_html(cls, src_path, violation_lines):
+    def load_formatted_snippets(cls, src_path, violation_lines):
         """
         Load snippets from the file at `src_path` and format
-        them as HTML.
+        them as HTML and as plain text.
+        Returns a dictionary containing the two types of formatting
+        results for code snippets.
 
         See `load_snippets()` for details.
         """
+
+        # load once...
         snippet_list = cls.load_snippets(src_path, violation_lines)
-        return [snippet.html() for snippet in snippet_list]
+
+        # ...render twice in different formats
+        return {
+            "html": [snippet.html() for snippet in snippet_list],
+            "markdown": [snippet.markdown() for snippet in snippet_list],
+        }
+
+    @classmethod
+    def load_contents(cls, src_path):
+        try:
+            with openpy(GitPathTool.relative_path(src_path)) as src_file:
+                contents = src_file.read()
+        except (SyntaxError, UnicodeDecodeError):
+            # this tool was originally written with python in mind.
+            # for processing non python files encoded in anything other than ascii or utf-8 that
+            # code wont work
+            with open(GitPathTool.relative_path(src_path), "rb") as src_file:
+                contents = src_file.read()
+
+        if isinstance(contents, bytes):
+            encoding = chardet.detect(contents).get("encoding", "utf-8")
+            try:
+                contents = contents.decode(encoding)
+            except UnicodeDecodeError:
+                pass
+
+        if isinstance(contents, bytes):
+            # We failed to decode the file.
+            # if this is happening a lot I should just bite the bullet
+            # and write a parameter to let people list their file encodings
+            print(
+                "Warning: I was not able to decode your src file. "
+                "I can continue but code snippets in the final report may look wrong"
+            )
+            contents = contents.decode("utf-8", "replace")
+        return contents
 
     @classmethod
     def load_snippets(cls, src_path, violation_lines):
@@ -150,27 +224,21 @@ class Snippet(object):
 
         Raises an `IOError` if the file could not be loaded.
         """
-        # Load the contents of the file
-        with openpy(GitPathTool.relative_path(src_path)) as src_file:
-            contents = src_file.read()
-
-        # Convert the source file to unicode (Python < 3)
-        if isinstance(contents, six.binary_type):
-            contents = contents.decode('utf-8', 'replace')
+        contents = cls.load_contents(src_path)
 
         # Construct a list of snippet ranges
-        src_lines = contents.split('\n')
+        src_lines = contents.split("\n")
         snippet_ranges = cls._snippet_ranges(len(src_lines), violation_lines)
 
         # Parse the source into tokens
-        token_stream = cls._parse_src(contents, src_path)
+        token_stream, lexer = cls._parse_src(contents, src_path)
 
         # Group the tokens by snippet
         token_groups = cls._group_tokens(token_stream, snippet_ranges)
 
         return [
-            Snippet(tokens, src_path, start, violation_lines)
-            for (start, _), tokens in sorted(token_groups.items())
+            Snippet(tokens, src_path, start, end, violation_lines, lexer.name)
+            for (start, end), tokens in sorted(token_groups.items())
         ]
 
     @classmethod
@@ -193,7 +261,7 @@ class Snippet(object):
         # the source file when lexing.
         lexer.stripnl = False
 
-        return pygments.lex(src_contents, lexer)
+        return pygments.lex(src_contents, lexer), lexer
 
     @classmethod
     def _group_tokens(cls, token_stream, range_list):
@@ -220,7 +288,7 @@ class Snippet(object):
         """
 
         # Create a map from ranges (start/end tuples) to tokens
-        token_map = dict((rng, []) for rng in range_list)
+        token_map = {rng: [] for rng in range_list}
 
         # Keep track of the current line number; we will
         # increment this as we encounter newlines in token values
@@ -232,22 +300,22 @@ class Snippet(object):
             # we need to split it up and check whether
             # each line within the token is within one
             # of our ranges.
-            if '\n' in val:
-                val_lines = val.split('\n')
+            if "\n" in val:
+                val_lines = val.split("\n")
 
                 # Check if the tokens match each range
-                for (start, end), filtered_tokens in six.iteritems(token_map):
+                for (start, end), filtered_tokens in token_map.items():
 
                     # Filter out lines that are not in this range
                     include_vals = [
-                        val_lines[i] for i in
-                        range(0, len(val_lines))
+                        val_lines[i]
+                        for i in range(0, len(val_lines))
                         if i + line_num in range(start, end + 1)
                     ]
 
                     # If we found any lines, store the tokens
                     if len(include_vals) > 0:
-                        token = (ttype, '\n'.join(include_vals))
+                        token = (ttype, "\n".join(include_vals))
                         filtered_tokens.append(token)
 
                 # Increment the line number
@@ -258,7 +326,7 @@ class Snippet(object):
             # If we're in the line range, add it
             else:
                 # Check if the tokens match each range
-                for (start, end), filtered_tokens in six.iteritems(token_map):
+                for (start, end), filtered_tokens in token_map.items():
 
                     # If we got a match, store the token
                     if line_num in range(start, end + 1):
@@ -306,8 +374,7 @@ class Snippet(object):
                     # Expand to include extra context, but not after last line
                     snippet_end = line_num - lines_since_last_violation
                     snippet_end = min(
-                        num_src_lines,
-                        snippet_end + cls.NUM_CONTEXT_LINES
+                        num_src_lines, snippet_end + cls.NUM_CONTEXT_LINES
                     )
                     current_range = (current_range[0], snippet_end)
 
@@ -337,6 +404,8 @@ class Snippet(object):
         than or equal to `start_line`; otherwise, they will
         be excluded from the list.
         """
-        return [line_num - start_line + 1
-                for line_num in line_num_list
-                if line_num >= start_line]
+        return [
+            line_num - start_line + 1
+            for line_num in line_num_list
+            if line_num >= start_line
+        ]
